# -*- coding: utf-8 -*-


class Proto:
    ADD_REVIEWER = "add-reviewer"
    DELETE_REVIEWER = "delete-reviewer"
    ADD_ATTENTION = "add-attention"
    REMOVE_ATTENTION = "remove-attention"
    APPROVE_CHANGE = "approve-change"
    SUBMIT_CHANGE = "submit-change"


class Separator:
    ACTION = ":"
    CONTENT = ","
    GROUP = " "
    LABEL = "="
